@@ -105,6 +105,7 @@ entree fonctions[]={
 {"boundcf",21,(void *)gboundcf,4,0},
 {"boundfact",21,(void *)boundfact,4,0},
 {"box",35,(void *)rectbox,10,0},
+{"buchcertify",10,(void *)certifybuchall,6,0},
 {"buchfu",1,(void *)buchfu,6,0},
 {"buchgen",92,(void *)buchall,6,0},
 {"buchgenforcefu",95,(void *)buchall,6,0},
@@ -114,6 +115,8 @@ entree fonctions[]={
 {"buchinitforcefu",89,(void *)buchall,6,0},
 {"buchinitfu",90,(void *)buchall,6,0},
 {"buchnarrow",1,(void *)buchnarrow,6,0},
+{"buchray",2,(void *)buchray,6,0},
+{"buchrayinit",2,(void *)buchrayinit,6,0},
 {"buchreal",97,(void *)buchreal,4,0},
 {"bytesize",10,(void *)taille2,2,0},
 {"ceil",1,(void *)gceil,2,0},
@@ -286,6 +289,8 @@ entree fonctions[]={
 {"isprime",1,(void *)gisprime,4,0},
 {"isprincipal",2,(void *)isprincipal,6,0},
 {"isprincipalgen",2,(void *)isprincipalgen,6,0},
+{"isprincipalray",2,(void *)isprincipalray,6,0},
+{"isprincipalraygen",2,(void *)isprincipalraygen,6,0},
 {"ispsp",1,(void *)gispsp,4,0},
 {"isqrt",1,(void *)racine,4,0},
 {"isset",10,(void *)isvecset,8,0},
@@ -353,6 +358,7 @@ entree fonctions[]={
 {"min",2,(void *)gmin,1,0},
 {"minideal",3,(void *)minideal,6,0},
 {"minim",33,(void *)minim,8,0},
+{"minim2",23,(void *)minim2,8,0},
 {"mod",25,(void *)gmodulcp,2,0},
 {"modp",25,(void *)gmodulo,2,0},
 {"modreverse",1,(void *)polymodrecip,6,0},
@@ -395,6 +401,7 @@ entree fonctions[]={
 {"plot",37,(void *)plot,10,0},
 {"ploth",37,(void *)ploth,10,0},
 {"ploth2",37,(void *)ploth2,10,0},
+{"plothmult",37,(void *)plothmult,10,0},
 {"plothraw",2,(void *)plothraw,10,0},
 {"pnqn",1,(void *)pnqn,4,0},
 {"point",35,(void *)rectpoint,10,0},
@@ -404,6 +411,9 @@ entree fonctions[]={
 {"polred",1,(void *)polred,6,0},
 {"polred2",1,(void *)polred2,6,0},
 {"polredabs",1,(void *)polredabs,6,0},
+{"polredabs2",1,(void *)polredabs2,6,0},
+{"polredabsall",1,(void *)polredabsall,6,0},
+{"polredabsfast",1,(void *)polredabsfast,6,0},
 {"polsym",21,(void *)polsym,7,0},
 {"polvar",1,(void *)gpolvar,2,0},
 {"poly",14,(void *)gtopoly,2,0},
@@ -505,6 +515,7 @@ entree fonctions[]={
 {"smallpolred2",1,(void *)smallpolred2,6,0},
 {"smith",1,(void *)smith,8,0},
 {"smith2",1,(void *)smith2,8,0},
+{"smithpol",1,(void *)gsmith,8,0},
 {"solve",37,(void *)zbrent,9,0},
 {"sort",1,(void *)sort,8,0},
 {"sqr",1,(void *)gsqr,3,0},
@@ -514,6 +525,7 @@ entree fonctions[]={
 {"string",57,(void*)rectstring,10,0},
 {"sturm",10,(void *)sturm,7,0},
 {"sturmpart",30,(void *)sturmpart,7,0},
+{"subcyclo",2,(void *)subcyclo,6,0},
 {"subell",3,(void *)subell,5,0},
 {"subst",26,(void *)gsubst,7,0},
 {"sum",48,(void *)somme,9,0},
@@ -556,8 +568,12 @@ entree fonctions[]={
 {"zell",2,(void *)zell,5,0},
 {"zeta",1,(void *)gzeta,3,0},
 {"zetak",2,(void *)gzetak,6,0},
+{"zideallog",3,(void *)zideallog,6,0},
+{"zidealstar",2,(void *)zidealstar,6,0},
+{"zidealstarinit",2,(void *)zidealstarinit,6,0},
 {"znstar",1,(void *)znstar,4,0},
-{"zzzz",1,(void *)factor,4,0}
+{"zzzz",2,(void *)zidealstarinitold,6,0},
+{"zzzzz",10,(void *)certifybuchall,8,0}
 };
 
 long    NUMFUNC=sizeof(fonctions)/sizeof(entree);
@@ -591,7 +607,11 @@ seq(void)
 GEN
 expr(void)
 {
+#ifdef __cplusplus
+  typedef GEN (*PFGEN)(...);
+#else
   typedef GEN (*PFGEN)();
+#endif
   PFGEN func[4];
   GEN aux,e,e1,e2,e3;
   long niveau;
