@@ -448,7 +448,7 @@ gaddpex(GEN x, GEN y)
     }
     else e3=pvaluation(x,p,&p2);
     e2=signe((GEN)y[4])?e1+precp(y)-e3:e1-e3;
-    z[2]=(long)p;
+    z[2]=(long)pcopy(p);
     if(e2<=0) {z[3]=un;z[1]=evalprecp(0)+evalvalp(e3);}
     else
     {
