@@ -133,7 +133,7 @@ main(long argc, char **argv)
   
   init(parisize, primelimit);
   printversion();
-  pariputs("\nCopyright 1989-1994 by C. Batut, D. Bernardi, H. Cohen and M. Olivier\n\n");
+  pariputs("\nCopyright 1989-1995 by C. Batut, D. Bernardi, H. Cohen and M. Olivier\n\n");
   if(tmpparibuffsize) paribuffsize=tmpparibuffsize;
   buffer = (char *)malloc(paribuffsize);
 #ifdef _READLINE_H_
@@ -278,6 +278,8 @@ main(long argc, char **argv)
       pariputs(thestring);
     }
     if (z == gnil) continue;
+    if(tglobal>=STACKSIZE)
+      err(talker,"not enough g's in GP, please restart or \\k GP session");
     if(simplifyflag) 
     {
       if(isonstack(z))
