@@ -39,6 +39,7 @@ char *helpmessage[] =
   "boundcf(x,lmax)=continued fraction expansion of x with at most lmax terms",
   "boundfact(x,lim)=partial factorization of the integer x (using primes up\nto lim)",
   "box(w,x2,y2)=if the cursor is at position (x1,y1), draw a box with diagonal\n(x1,y1) and (x2,y2) in rectwindow w (cursor does not move)",
+  "buchcertify(bnf)=certify the correctness (i.e. remove the GRH) of the bnf\ndata output by buchinit or buchinitfu",
   "buchfu(bnf)=compute the fundamental units of the number field bnf output by\nbuchinit",
   "buchgen(P,...)=compute the structure of the class group and the\nregulator for the number field defined by the polynomial P. See manual\nfor the other parameters (which can be omitted)",
   "buchgenforcefu(P,...)=compute the structure of the class group, the regulator\na primitive root of unity and a system of fundamental units for the number\nfield defined by the polynomial P, and insist until the units are obtained.\nSee manual for the other parameters (which can be omitted)",
@@ -48,6 +49,8 @@ char *helpmessage[] =
   "buchinitforcefu(P,...)=compute the necessary data for future use in ideal and\nunit group computations, and insist on having fundamental units.\nSee manual for details",
   "buchinitfu(P,...)=compute the necessary data for future use in ideal and unit\ngroup computations, including fundamental units if they are not too large.\nSee manual for details",
   "buchnarrow(bnf)=given a big number field as output by buchinitxx, gives as a\n3-component vector the structure of the narrow class group",
+  "buchray(bnf,ideal)=given a big number field as output by buchinitfu (only) and\n an ideal or a 2-component row vector formed by an ideal and a list of R1\nzeros or ones representing a module, finds the ray class group structure\ncorresponding to this module",
+  "buchrayinit(bnf,ideal)=given a big number field as output by buchinitfu (only) and\n an ideal or a 2-component row vector formed by an ideal and a list of R1\nzeros or ones representing a module, initializes data for computing in the ray class group \ncorresponding to this module",
   "buchreal(D,...)=compute the structure of the class group and the\nregulator of the real quadratic field of discriminant D>0 in the wide sense.\nSee manual for the other parameters (which can be omitted)",
   "bytesize(x)=number of bytes occupied by the complete tree of the object x",
   "ceil(x)=ceiling of x=smallest integer>=x",
@@ -197,7 +200,7 @@ char *helpmessage[] =
   "indsort(x)=indirect sorting of the vector x",
   "initalg(x)=x being a nonconstant irreducible polynomial, gives the vector:\n[x,[r1,r2],discf,index,[M,MC,T2,T,different] (see manual),r1+r2 first roots,\nintegral basis, matrix of power basis in terms of integral basis,\nmultiplication table of basis]",
   "initalgred(x)=x being a nonconstant irreducible polynomial, finds (using\npolred) a simpler polynomial pol defining the same number field, and gives\nthe vector: [pol,[r1,r2],discf,index,[M,MC,T2,T,different] (see manual),\nr1+r2 first roots, integral basis, matrix of power basis in terms of\nintegral basis, multiplication table of basis]",
-  "initalgred2(P)=P being a nonconstant irreducible polynomial, gives the\npolymod mod(a,pol) equal to mod(x,P) where pol is as in initalgred",
+  "initalgred2(P)=P being a nonconstant irreducible polynomial, gives a two-element\nvector [nf,mod(a,pol)], where nf is as output by initalgred and mod(a,pol) is\na polymod equal to mod(x,P) and pol=nf[1]",
   "initell(x)=x being the vector [a1,a2,a3,a4,a6], gives the vector:\n[a1,a2,a3,a4,a6,b2,b4,b6,b8,c4,c6,delta,j,[e1,e2,e3],w1,w2,eta1,eta2,q,area]",
   "initell2(x)=x being the vector [a1,a2,a3,a4,a6], gives the vector:\n[a1,a2,a3,a4,a6,b2,b4,b6,b8,c4,c6,delta,j,[e1,e2,e3],w1,w2,eta1,eta2,q,area]",
   "initrect(w,x,y)=initialize rectwindow w to size x,y",
@@ -219,7 +222,9 @@ char *helpmessage[] =
   "isoncurve(e,x)=true(1) if x is on elliptic curve e, false(0) if not",
   "isprime(x)=true(1) if x is a strong pseudoprime for 10 random bases,\nfalse(0) if not",
   "isprincipal(bnf,x)=bnf being output by buchinit, gives the vector of exponents on the\nclass group generators of x. In particular x is principal if and only if the result is\nthe zero vector",
-  "isprincipalgen(bnf,x)=bnf being output by buchinit, gives [v,alpha], where v is\nthe vector of exponents on the class group generators and alpha is the\ngenerator of the resulting principal ideal. In particular x is principal\nif and only if v is the zero vector",
+  "isprincipalgen(bnf,x)=bnf being output by buchinit, gives [v,alpha,bitaccuracy],\nwhere v is the vector of exponents on the class group generators and alpha is the\ngenerator of the resulting principal ideal. In particular x is principal\nif and only if v is the zero vector",
+  "isprincipalray(bnf,x)=bnf being output by buchrayinit, gives the vector of exponents on the\nray class group generators of x. In particular x is principal if and only if the result is\nthe zero vector",
+  "isprincipalraygen(bnf,x)=bnf being output by buchrayinit, gives [v,alpha,bitaccuracy],\nwhere v is the vector of exponents on the class group generators and alpha is the\ngenerator of the resulting principal ideal. In particular x is principal\nif and only if v is the zero vector",
   "ispsp(x)=true(1) if x is a strong pseudoprime, false(0) if not",
   "isqrt(x)=integer square root of x (x integer)",
   "isset(x)=true(1) if x is a set (row vector with strictly increasing entries),\nfalse(0) if not",
@@ -287,6 +292,7 @@ char *helpmessage[] =
   "min(x,y)=minimum of x and y",
   "minideal(nf,ix,vdir)=minimum of the ideal ix in the direction vdir in the\nnumber field nf",
   "minim(x,bound,maxnum)=number of vectors of square norm <= bound, maximum\nnorm and list of vectors for the integral and definite quadratic form x;\nminimal non-zero vectors if bound=0",
+  "minim2(x,bound)=looks for vectors of square norm <= bound, return the first one and its norm",
   "mod(x,y)=creates the integer x modulo y on the PARI stack",
   "modp(x,y)=creates the integer x modulo y as a permanent object (on the heap)",
   "modreverse(x)=reverse polymod of the polymod x, if it exists",
@@ -329,6 +335,7 @@ char *helpmessage[] =
   "plot(X=a,b,expr)=crude plot of expression expr, X goes from a to b",
   "ploth(X=a,b,expr)=plot of expression expr, X goes from a to b in high\nresolution",
   "ploth2(X=a,b,[expr1,expr2])=plot of points [expr1,expr2], X goes from a to\nb in high resolution",
+  "plothmult(X=a,b,[expr1,...])=plot of expressions expr1,..., X goes from a to\nb in high resolution",
   "plothraw(listx,listy)=plot in high resolution points whose x (resp. y)\ncoordinates are in listx (resp. listy)",
   "pnqn(x)=[p_n,p_{n-1};q_n,q_{n-1}] corresponding to the continued fraction x",
   "point(w,x,y)=draw a point (and move cursor) at position x,y in rectwindow w",
@@ -337,7 +344,10 @@ char *helpmessage[] =
   "polint(xa,ya,x)=polynomial interpolation at x according to data vectors xa, ya",
   "polred(x)=reduction of the polynomial x (gives minimal polynomials only)",
   "polred2(x)=reduction of the polynomial x (gives elements and minimal\npolynomials)",
-  "polredabs(x)=smallest generating polynomial of the number field for the T2\nnorm on the roots",
+  "polredabs(x)=a smallest generating polynomial of the number field for the T2\nnorm on the roots, with smallest index for the minimal T2 norm",
+  "polredabs2(x)=gives [pol,a] where pol is as in polredabs, and alpha is the\nelement whose characteristic polynomial is pol",
+  "polredabsall(x)=complete list of the smallest generating polynomials of the\nnumber field for the T2 norm on the roots",
+  "polredabsfast(x)=a smallest generating polynomial of the number field for the T2\nnorm on the roots",  
   "polsym(x,n)=vector of symmetric powers of the roots of x up to n",
   "polvar(x)=main variable of object x. Gives p for p-adic x, error for scalars",  
   "poly(x,v)=convert x (usually a vector or a power series) into a polynomial\nwith variable v, starting with the leading coefficient",
@@ -439,6 +449,7 @@ char *helpmessage[] =
   "smallpolred2(x)=partial reduction of the polynomial x (gives elements and\nminimal polynomials)",
   "smith(x)=Smith normal form (i.e. elementary divisors) of the matrix x,\nexpressed as a vector",
   "smith2(x)=gives a two element vector [u,v] where u and v are matrices such\nthat u*x*v=smith(x)",
+  "smithpol(x)=Smith normal form (i.e. elementary divisors) of the matrix x\nwith polynomial coefficients, expressed as a vector",
   "solve(X=a,b,expr)=real root of expression expr (X between a and b), where\nexpr(a)*expr(b)<=0",
   "sort(x)=sort in ascending order of the vector x",
   "sqr(x)=square of x. NOT identical to x*x",
@@ -448,11 +459,12 @@ char *helpmessage[] =
   "string(w,x)=draw in rectwindow w the string corresponding to x, where x is\neither a string, or a number in R, written in format 9.3",
   "sturm(x)=number of real roots of the polynomial x",
   "sturmpart(x,a,b)=number of real roots of the polynomial x in the\ninterval (a,b]",
+  "subcylo(p,d)=finds an equation for the d-th degree subfield of Q(zeta_p),\nwhere p must be a prime power",
   "subell(e,z1,z2)=difference of the points z1 and z2 on elliptic curve e",
   "subst(x,y,z)=in expression x, replace the variable y by the expression z",
   "sum(x,X=a,b,expr)=x plus the sum (X goes from a to b) of expression expr",
-  "sumalt(X=a,expr)=Villegas's acceleration of alternating series expr,\nX starting at a",
-  "sumalt2(X=a,expr)=Euler-Van Wijngarten's acceleration of alternating series\nexpr, X starting at a",
+  "sumalt(X=a,expr)=Villegas-Zagier's acceleration of alternating series expr,\nX starting at a",
+  "sumalt2(X=a,expr)=Euler-Van Wijngaarden's acceleration of alternating series\nexpr, X starting at a",
   "suminf(X=a,expr)=infinite sum (X goes from a to infinity) of real or complex\nexpression expr",
   "sumpos(X=a,expr)=sum of positive series expr, the formal variable X\nstarting at a",
   "supplement(x)=supplement the columns of the matrix x to an invertible matrix",
@@ -490,6 +502,10 @@ char *helpmessage[] =
   "zell(e,z)=In the complex case, lattice point corresponding to the point z on\nthe elliptic curve e",
   "zeta(s)=Riemann zeta function at s",
   "zetak(nfz,s)=Dedekind zeta function of the number field nfz at s, where nfz\nis the vector computed by initzeta (NOT by initalg)",
+  "zideallog(nf,x,bid)=if bid is a big ideal as given by zidealstarinit, gives the\nvector of exponents on the generators bid[1][3]",
+  "zidealstar(nf,I)=3-component vector v, giving the structure of (Z_K/I)^*. v[1] is\n the order (i.e. phi(I)), v[2] is a vector of cyclic components, and v[3]\n is a vector giving the corresponding generators",
+  "zidealstarinit(nf,I)=6-component vector [I,v,fa,f2,U,V] where v is as in zidealstar\nfa is the prime ideal factorisation of I and f2, U and V are technical but\nessential to work in (Z_K/I)^*",
   "znstar(n)=3-component vector v, giving the structure of (Z/nZ)^*. v[1] is\n the order (i.e. phi(n)), v[2] is a vector of cyclic components, and v[3]\n is a vector giving the corresponding generators",
-  "zzzz(...)=can be any function which is being tested. For developing purposes only"
+  "zzzz(...)=can be any function which is being tested. For developing purposes only",
+  "zzzzz(...)=can be any function which is being tested. For developing purposes only"
 };
