@@ -23,11 +23,46 @@ extern "C" {
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xos.h>
+int fork();
 #ifdef __cplusplus
 }
 #endif
 
 
+Colormap PARI_Colormap;		/* The Colormap We Are Using */
+XColor  *PARI_Colors;
+XColor  *PARI_ExactColors;
+
+static char *PARI_DefaultColors[8] = {
+  " ",
+  "sienna", /* Courbe impaires dans plothmult */
+  "cornsilk",   
+  "red",        /* Courbes, ou courbes paires dans plothmult */
+  "black",                    
+  "grey",                     
+  "blue",         /* axes */
+  "gainsboro",              
+};
+
+void
+PARI_ColorSetUp(Display *display, char **Colors, int n)
+{
+  static int NotInitialised = 1;
+  if (NotInitialised)
+  {
+    int i;
+    NotInitialised = 0;
+    PARI_Colormap = DefaultColormap(display, 0);
+    PARI_Colors = (XColor *) malloc((n + 1) * sizeof(XColor));
+    PARI_ExactColors = (XColor *) malloc((n + 1) * sizeof(XColor));
+    for (i = 1; i < n; i++)
+    {
+      XAllocNamedColor(display, PARI_Colormap, Colors[i],
+		       &PARI_ExactColors[i], &PARI_Colors[i]);
+    }
+  }
+}
+
 #define IDEC 90
 #define JDEC 5
 
@@ -66,12 +101,27 @@ xerror()
   err(ploter1);return 0;
 }
 
+double
+tickmarks(double a)
+{
+  long k;
+  double m,e;
+  
+  if(a<=0) return 0.0;
+  k=(long)(log(a)/log(10));e=pow(10.0,(double)k);
+  m=a/e;
+  if(m<=2) return 0.1*e;
+  if(m<=5) return 0.2*e;
+  return 0.5*e;
+}
+
 GEN
 ploth(entree *ep, GEN a, GEN b, char *ch, long prec)
 {
-  long av = avma,av2,jz,i,sig,is,js,iz;
+  long av = avma,av2,jz,i,sig,is,js,iz = 0;
   GEN p1,ysml,ybig,x,diff,dyj,dx,y[2000];
   char c1[20], c2[20], c3[20], c4[20];
+  double sht,svt;
   XPoint points[2000];
 
   Display *display;
@@ -82,7 +132,9 @@ ploth(entree *ep, GEN a, GEN b, char *ch, long prec)
   XFontStruct *font_info;
   int screen, width, height;
   
-  sig=gcmp(b,a); if(!sig) return gnil;
+  if(fork()) return gnil;
+
+  sig=gcmp(b,a); if(!sig) {abort();return gnil;}
   if(sig<0) {x=a;a=b;b=x;}
   
   if (!(display = XOpenDisplay(NULL))) err(ploter2);
@@ -90,6 +142,7 @@ ploth(entree *ep, GEN a, GEN b, char *ch, long prec)
 
   XSetErrorHandler((XErrorHandler)xerror);
   XSetIOErrorHandler((XIOErrorHandler)xerror);
+  PARI_ColorSetUp(display,PARI_DefaultColors, 8);
 
   screen = DefaultScreen(display);
   width = DisplayWidth(display, screen) - 40;
@@ -111,6 +164,7 @@ ploth(entree *ep, GEN a, GEN b, char *ch, long prec)
   diff = gsub(ybig, ysml);
   if(gcmp0(diff)) {ybig=gaddsg(1,ybig);diff=gun;}
   dyj = gdivsg(js, diff); av2 = avma;
+  sht=tickmarks(rtodbl(gsub(b,a)));svt=tickmarks(rtodbl(diff));
   for(i = 0; i <= is; i++)
   {
     points[i].x = IDEC + i;
@@ -139,30 +193,161 @@ ploth(entree *ep, GEN a, GEN b, char *ch, long prec)
 
   gc = XCreateGC(display, win, 0, NULL);
   XSetFont(display, gc, font_info->fid);
-  XSetForeground(display, gc, BlackPixel(display, screen)); 
 
   XMapWindow(display, win);
 
   for(;;)
   {
+    XSetForeground(display, gc, BlackPixel(display, screen));
     XNextEvent(display, &report);
     if (report.type != Expose) break;
     while (XCheckTypedEvent(display, Expose, &report));
     XDrawRectangle(display, win, gc, IDEC, JDEC, is, js);
-    XDrawLine(display, win, gc, IDEC, jz, IDEC + is, jz);
-    if (iz) XDrawLine(display, win, gc, iz, JDEC, iz, js + JDEC);
     XDrawString(display, win, gc, 0, 15, c1, strlen(c1));
     XDrawString(display, win, gc, 0, height - 30, c2, strlen(c2));
     XDrawString(display, win, gc, IDEC - 20, height - 5, c3, strlen(c3));
     XDrawString(display, win, gc, width - 90, height - 5, c4, strlen(c4));
+    XSetForeground(display, gc, PARI_Colors[6].pixel);
+    XDrawLine(display, win, gc, IDEC, jz, IDEC + is, jz);
+    if (iz) XDrawLine(display, win, gc, iz, JDEC, iz, js + JDEC);
+    XSetForeground(display, gc, PARI_Colors[3].pixel);
     XDrawLines(display, win, gc, points, is+1, 0);
   }
   XUnloadFont(display, font_info->fid);
   XFreeGC(display, gc);
   XCloseDisplay(display);
   avma = av;
+  killvalue(ep);abort();return gnil;
+}
+
+
+GEN
+plothmult(entree *ep, GEN a, GEN b, char *ch, long prec)
+{
+  long av = avma,av2,jz,i,j,sig,is,js,iz = 0,nc;
+  GEN p1,ysml,ybig,x,diff,dyj,dx,y[2000];
+  char c1[20], c2[20], c3[20], c4[20];
+  double sht,svt;
+  XPoint **pointvec;
+  
+  Display *display;
+  Window win;
+  XSizeHints size_hints;
+  XEvent report;
+  GC gc;
+  XFontStruct *font_info;
+  int screen, width, height;
+  
+  if(fork()) return gnil;
+
+  sig=gcmp(b,a); if(!sig) {abort();return gnil;}
+  if(sig<0) {x=a;a=b;b=x;}
+  
+  if (!(display = XOpenDisplay(NULL))) err(ploter2);
+  if (!(font_info = XLoadQueryFont(display, "9x15"))) err(ploter3);
+
+  XSetErrorHandler((XErrorHandler)xerror);
+  XSetIOErrorHandler((XIOErrorHandler)xerror);
+  PARI_ColorSetUp(display,PARI_DefaultColors, 8);
+
+  screen = DefaultScreen(display);
+  width = DisplayWidth(display, screen) - 40;
+  height = DisplayHeight(display, screen) - 60;
+  is = width - IDEC - 5;
+  js = height - 25;
+
+  newvalue(ep,cgetr(prec)); x=(GEN)(ep->value);
+  gaffect(a,x);p1=lisexpr(ch);
+  if(typ(p1)!=17) err(talker,"not a row vector in plothmult");
+  nc=lg(p1);if(nc<=1) {avma=av;abort();return gnil;}
+  pointvec=(XPoint **)malloc(nc*sizeof(XPoint*));
+  if(!pointvec) err(talker,"not enough memory in plothmult");
+  for(j=1;j<nc;j++)
+  {
+    pointvec[j]=(XPoint*)malloc(2000*sizeof(XPoint));
+    if(!pointvec[j]) err(talker,"not enough memory in plothmult");
+  }
+  dx=gdivgs(gsub(b,a),is);
+  ysml=cgetr(3);ybig=cgetr(3);affsr(0,ysml);affsr(0,ybig);
+  for(i=0;i<=is;i++)
+  {
+    p1=cgetg(nc,17);y[i]=p1;
+    for(j=1;j<nc;j++) p1[j]=lgetr(3);
+  }
+  av2=avma;
+  for(i=0;i<=is;i++)
+  {
+    gaffect(lisexpr(ch), y[i]);
+    if(gcmp(p1=vecmin(y[i]), ysml) < 0) gaffect(p1,ysml);
+    else if(gcmp(p1=vecmax(y[i]), ybig) > 0) gaffect(p1,ybig);
+    gaddz(x,dx,x);avma=av2;
+  }
+  diff = gsub(ybig, ysml);
+  if(gcmp0(diff)) {gaddsgz(1,ybig,ybig);diff=gun;}
+  dyj = gdivsg(js, diff); av2 = avma;
+  sht=tickmarks(rtodbl(gsub(b,a)));svt=tickmarks(rtodbl(diff));
+  for(i = 0; i <= is; i++)
+  {
+    for(j=1;j<nc;j++)
+    {
+      pointvec[j][i].x = IDEC + i;
+      pointvec[j][i].y = JDEC + itos(ground(gmul(gsub(ybig,(GEN)(y[i][j])),dyj)));
+      avma=av2;
+    }
+  }
+  jz = JDEC + itos(ground(gmul(ybig,dyj)));
+  if(gsigne(a)*gsigne(b)<0)
+    iz = IDEC + itos(ground(gmulsg(is, gdiv(a, gsub(a, b)))));
+  p1=cgetr(4); gaffect(ybig,p1); sprintf(c1,"%9.3f",rtodbl(p1));
+  gaffect(ysml,p1); sprintf(c2,"%9.3f",rtodbl(p1));
+  gaffect(a,p1); sprintf(c3,"%9.3f",rtodbl(p1));
+  gaffect(b,p1); sprintf(c4,"%9.3f",rtodbl(p1));
+
+  win = XCreateSimpleWindow(display, RootWindow(display, screen), 0, 0, width, height, 4, BlackPixel(display, screen), WhitePixel(display, screen));
+
+  size_hints.flags = PPosition | PSize;
+  size_hints.x = 0;
+  size_hints.y = 0;
+  size_hints.width = width;
+  size_hints.height = height;
+
+  XSetStandardProperties(display, win, "ploth", NULL, None, NULL, 0, &size_hints);
+
+  XSelectInput(display, win, ExposureMask | ButtonPressMask);
+
+  gc = XCreateGC(display, win, 0, NULL);
+  XSetFont(display, gc, font_info->fid);
+
+  XMapWindow(display, win);
+
+  for(;;)
+  {
+    XSetForeground(display, gc, BlackPixel(display, screen));
+    XNextEvent(display, &report);
+    if (report.type != Expose) break;
+    while (XCheckTypedEvent(display, Expose, &report));
+    XDrawRectangle(display, win, gc, IDEC, JDEC, is, js);
+    XDrawString(display, win, gc, 0, 15, c1, strlen(c1));
+    XDrawString(display, win, gc, 0, height - 30, c2, strlen(c2));
+    XDrawString(display, win, gc, IDEC - 20, height - 5, c3, strlen(c3));
+    XDrawString(display, win, gc, width - 90, height - 5, c4, strlen(c4));
+    XSetForeground(display, gc, PARI_Colors[6].pixel);
+    XDrawLine(display, win, gc, IDEC, jz, IDEC + is, jz);
+    if (iz) XDrawLine(display, win, gc, iz, JDEC, iz, js + JDEC);
+    for(j=1;j<nc;j++)
+    {
+      if(j&1) XSetForeground(display, gc, PARI_Colors[1].pixel);
+      else XSetForeground(display, gc, PARI_Colors[3].pixel);
+      XDrawLines(display, win, gc, pointvec[j], is+1, 0);
+    }
+  }
+  XUnloadFont(display, font_info->fid);
+  XFreeGC(display, gc);
+  XCloseDisplay(display);
+  avma = av;
   killvalue(ep);
-  return gnil;
+  for(j=1;j<nc;j++) free(pointvec[j]);free(pointvec);
+  abort();return gnil;
 }
 
 #define NBPOINTS 1500
@@ -183,7 +368,9 @@ ploth2(entree *ep, GEN a, GEN b, char *ch, long prec)
   XFontStruct *font_info;
   int screen, width, height;
   
-  sig=gcmp(b,a); if(!sig) return gnil;
+  if(fork()) return gnil;
+
+  sig=gcmp(b,a); if(!sig) {abort();return gnil;}
   if(sig<0) {t=a;a=b;b=t;}
   
   if (!(display = XOpenDisplay(NULL))) err(ploter2);
@@ -191,6 +378,7 @@ ploth2(entree *ep, GEN a, GEN b, char *ch, long prec)
 
   XSetErrorHandler((XErrorHandler)xerror);
   XSetIOErrorHandler((XIOErrorHandler)xerror);
+  PARI_ColorSetUp(display,PARI_DefaultColors, 8);
 
   screen = DefaultScreen(display);
   width = DisplayWidth(display, screen) - 40;
@@ -249,22 +437,24 @@ ploth2(entree *ep, GEN a, GEN b, char *ch, long prec)
 
   gc = XCreateGC(display, win, 0, NULL);
   XSetFont(display, gc, font_info->fid);
-  XSetForeground(display, gc, BlackPixel(display, screen));
 
   XMapWindow(display, win);
 
   for(;;)
   {
+    XSetForeground(display, gc, BlackPixel(display, screen));
     XNextEvent(display, &report);
     if (report.type != Expose) break;
     while (XCheckTypedEvent(display, Expose, &report));
     XDrawRectangle(display, win, gc, IDEC, JDEC, width - IDEC - 5, js);
-    XDrawLine(display, win, gc, iz, JDEC, iz, js + JDEC);
-    XDrawLine(display, win, gc, IDEC, jz, width - 5, jz);
     XDrawString(display, win, gc, 0, 15, c1, strlen(c1));
     XDrawString(display, win, gc, 0, height - 30, c2, strlen(c2));
     XDrawString(display, win, gc, IDEC - 20, height - 5, c3, strlen(c3));
     XDrawString(display, win, gc, width - 90, height - 5, c4, strlen(c4));
+    XSetForeground(display, gc, PARI_Colors[6].pixel);
+    XDrawLine(display, win, gc, iz, JDEC, iz, js + JDEC);
+    XDrawLine(display, win, gc, IDEC, jz, width - 5, jz);
+    XSetForeground(display, gc, PARI_Colors[3].pixel);
     XDrawLines(display, win, gc, points, NBPOINTS, 0);
   }
   XUnloadFont(display, font_info->fid);
@@ -272,7 +462,7 @@ ploth2(entree *ep, GEN a, GEN b, char *ch, long prec)
   XCloseDisplay(display);
   avma = av;
   killvalue(ep);
-  return gnil;
+  abort();return gnil;
 }
 	
 GEN
@@ -291,11 +481,13 @@ plothraw(GEN listx, GEN listy)
   XFontStruct *font_info;
   int screen, width, height;
   
+  if(fork()) return gnil;
+
   if((typ(listx)<17)||(typ(listx)>18)||(typ(listy)<17)||(typ(listy)>18))
     err(ploter4);
   lx=lg(listx);
   if(lg(listy)!=lx) err(ploter5);
-  if(lx==1) return gnil;
+  if(lx==1) {abort();return gnil;}
   points = (XPoint*)malloc(lx*sizeof(XPoint));
   if(!points) err(ploter6);
   if (!(display = XOpenDisplay(NULL))) err(ploter2);
@@ -303,6 +495,7 @@ plothraw(GEN listx, GEN listy)
 
   XSetErrorHandler((XErrorHandler)xerror);
   XSetIOErrorHandler((XIOErrorHandler)xerror);
+  PARI_ColorSetUp(display,PARI_DefaultColors, 8);
 
   screen = DefaultScreen(display);
   width = DisplayWidth(display, screen) - 40;
@@ -369,12 +562,12 @@ plothraw(GEN listx, GEN listy)
 
   gc = XCreateGC(display, win, 0, NULL);
   XSetFont(display, gc, font_info->fid);
-  XSetForeground(display, gc, BlackPixel(display, screen)); 
 
   XMapWindow(display, win);
 
   for(;;)
   {
+    XSetForeground(display, gc, BlackPixel(display, screen)); 
     XNextEvent(display, &report);
     if (report.type != Expose) break;
     while (XCheckTypedEvent(display, Expose, &report));
@@ -383,13 +576,14 @@ plothraw(GEN listx, GEN listy)
     XDrawString(display, win, gc, 0, height - 30, c2, strlen(c2));
     XDrawString(display, win, gc, IDEC - 20, height - 5, c3, strlen(c3));
     XDrawString(display, win, gc, width - 90, height - 5, c4, strlen(c4));
+    XSetForeground(display, gc, PARI_Colors[3].pixel);
     XDrawPoints(display, win, gc, points, lx-1, 0);
   }
   XUnloadFont(display, font_info->fid);
   XFreeGC(display, gc);
   XCloseDisplay(display);
   free(points);avma = av;
-  return gnil;
+  abort();return gnil;
 }
 
 GEN
@@ -411,13 +605,17 @@ rectdraw(GEN list)
   XSegment *segments;
   XRectangle *rectangles;
 
+  if(fork()) return gnil;
+
   if(typ(list)!=17) err(rploter3);
   n=lg(list)-1;if(n%3) err(rploter4);
-  n=n/3;if(!n) return gnil;
+  n=n/3;if(!n) {abort();return gnil;}
   if (!(display = XOpenDisplay(NULL))) err(ploter2);
   if (!(font_info = XLoadQueryFont(display, "9x15"))) err(ploter3);
   XSetErrorHandler((XErrorHandler)xerror);
   XSetIOErrorHandler((XIOErrorHandler)xerror);
+  PARI_ColorSetUp(display,PARI_DefaultColors, 8);
+
   screen = DefaultScreen(display);
   width = DisplayWidth(display, screen) - 40;
   height = DisplayHeight(display, screen) - 60; 
@@ -508,10 +706,10 @@ rectdraw(GEN list)
   XSelectInput(display, win, ExposureMask | ButtonPressMask);
   gc = XCreateGC(display, win, 0, NULL);
   XSetFont(display, gc, font_info->fid);
-  XSetForeground(display, gc, BlackPixel(display, screen)); 
   XMapWindow(display, win);
   for(;;)
   {
+    XSetForeground(display, gc, PARI_Colors[3].pixel);
     XNextEvent(display, &report);
     if (report.type != Expose) break;
     while (XCheckTypedEvent(display, Expose, &report));
@@ -527,5 +725,5 @@ rectdraw(GEN list)
   free(numpoints);for(i=0;i<nd[5];i++) free(lines[i]);
   free(lines);free(texts);free(numtexts);free(xtexts);free(ytexts);
   XCloseDisplay(display);
-  avma = av;return gnil;
+  avma = av;abort();return gnil;
 }
